package de.hmmh.pmt;

import com.fasterxml.jackson.databind.ObjectMapper;

import de.hmmh.pmt.db.*;
import de.hmmh.pmt.dtos.*;
import de.hmmh.pmt.employee.ApiClientFactory;
import de.hmmh.pmt.employee.api.EmployeeControllerApi;
import de.hmmh.pmt.employee.dtos.EmployeeResponseDTO;
import de.hmmh.pmt.oas.DefaultApi;
import de.hmmh.pmt.util.ApiTools;
import de.hmmh.pmt.util.Mapper;
import de.hmmh.pmt.util.Validator;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestClientException;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

@Controller
@RequestMapping("${openapi.projectManagement.base-path:/api/v1}")
public class ApiController implements DefaultApi {
    @Autowired
    private Mapper mapper;
    @Autowired
    private ApiClientFactory apiClientFactory;
    @Autowired
    private Validator validator;
    @Autowired
    private ApiTools apiTools;
    @Autowired
    private ProjectRepository projectRepository;
    @Autowired
    AllocationRepository allocationRepository;

    @Override
    public Optional<ObjectMapper> getObjectMapper() {
        return Optional.empty();
    }

    @Override
    public Optional<HttpServletRequest> getRequest() {
        return Optional.empty();
    }

    @Override
    public ResponseEntity<Void> deleteProject(Long id) {
        if (!projectRepository.existsById(id)) {
            return ResponseEntity.notFound().build();
        }

        projectRepository.deleteById(id);
        return ResponseEntity.noContent().build();
    }

    @Override
    public ResponseEntity<GetAllProjectsDTO> getAllProjects() {
        GetAllProjectsDTO response = new GetAllProjectsDTO();

        for (Project project : this.projectRepository.findAll()) {
            ProjectInfo projectInfo = new ProjectInfo();
            projectInfo.setId(project.getId());
            projectInfo.setName(project.getName());
            response.add(projectInfo);
        }

        return ResponseEntity.ok(response);
    }

    @Override
    public ResponseEntity<Void> updateProject(Long id, UpdateProjectDTO body) {
        Optional<Project> optionalProject = projectRepository.findById(id);
        if (optionalProject.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        Project project = optionalProject.get();

        if (project.getName().equals(body.getName()) && projectRepository.existsByName(body.getName())) {
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }

        if (project.getAdministratorId().equals(body.getAdministratorId())) {
            HttpStatus status = apiTools.checkEmployeeExists(body.getAdministratorId()).status();
            if (status != HttpStatus.OK) {
                return new ResponseEntity<>(status);
            }
        }

        Project finalProject = mapper.map(project, body);
        if (!validator.isValidProject(project)) {
            return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
        }

        if (validator.areAllocationTimeRangesOverlapping(
                project.getStart(),
                project.getPlannedEnd(),
                allocationRepository
                        .findAllByProjectId(finalProject.getId())
                        .stream()
                        .map(Allocation::getEmployeeId)
                        .flatMap(employeeId -> allocationRepository.findAllByEmployeeId(employeeId).stream())
                        .filter(employeeAllocation -> employeeAllocation.getProjectId().equals(finalProject.getId()))
                        .toList()
        )){
            return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
        }

        projectRepository.save(project);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Override
    public ResponseEntity<CreatedProjectDTO> createProject(CreateProjectDTO body) {
        if (projectRepository.existsByName(body.getName())) {
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }

        HttpStatus status = apiTools.checkEmployeeExists(body.getAdministratorId()).status();
        if (status != HttpStatus.OK) {
            return new ResponseEntity<>(status);
        }

        Project project = mapper.map(body);
        if (!validator.isValidProject(project)) {
            return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
        }
        projectRepository.save(project);

        CreatedProjectDTO response = mapper.map(project);
        return new ResponseEntity<>(response, HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<Void> addEmployee(Long id, AddEmployeeDTO body) {
        Optional<Project> optionalProject = projectRepository.findById(id);
        if (optionalProject.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        Project project = optionalProject.get();


        ApiTools.CheckEmployeeRecord employeeRecord = apiTools.checkEmployeeExists(body.getEmployeeId());
        if (employeeRecord.status() != HttpStatus.OK) {
            return new ResponseEntity<>(employeeRecord.status());
        }
        EmployeeResponseDTO employee = employeeRecord.employee();


        if (employee.getSkillSet()
                .stream()
                .noneMatch(qualification -> qualification.getId().equals(body.getQualificationId()))) {
            return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
        }

        if (validator.areAllocationTimeRangesOverlapping(
                project.getStart(),
                project.getPlannedEnd(),
                allocationRepository.findAllByEmployeeId(body.getEmployeeId())
        )) {
            return new ResponseEntity<>(HttpStatus.UNPROCESSABLE_ENTITY);
        }

        Allocation allocation = new Allocation();
        allocation.setEmployeeId(employee.getId());
        allocation.setRole(body.getQualificationId());
        allocation.setProject(project);
        allocationRepository.save(allocation);

        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Override
    public ResponseEntity<Void> completeProject(Long id) {
        Optional<Project> optionalProject = projectRepository.findById(id);
        if (optionalProject.isEmpty()) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        Project project = optionalProject.get();

        project.setRealEnd(LocalDateTime.now());
        projectRepository.save(project);
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Override
    public ResponseEntity<Void> removeEmployeeFromProject(Long id, Long employeeId){
       Optional<Allocation> allocation = allocationRepository.findById(new AllocationId(id, employeeId));
        if (allocation.isEmpty()){
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        allocationRepository.delete(allocation.get());
        return new ResponseEntity<>(HttpStatus.NO_CONTENT);
    }

    @Override
    public ResponseEntity<ProjectEmployeesDTO> getAListOfAllEmployeesFromASpecificProject(Long id) {
        if (!projectRepository.existsById(id)) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        ProjectEmployeesDTO dto = new ProjectEmployeesDTO();
        List<Allocation> allocationsByProject = allocationRepository.findAllByProjectId(id);
        if (allocationsByProject.isEmpty()) {
            return new ResponseEntity<>(dto, HttpStatus.OK);
        }

        Set<Long> employeeIds = allocationsByProject.stream()
                .map(Allocation::getEmployeeId)
                .collect(Collectors.toSet());

        try {
            List<Employee> employees = apiClientFactory.getEmployeeApi().findAll1().stream()
                    .filter(employeeResponseDTO -> employeeIds.contains(employeeResponseDTO.getId()))
                    .map(mapper::map)
                    .toList();
            dto.setEmployees(employees);
            return new ResponseEntity<>(dto, HttpStatus.OK);
        }  catch (RestClientException exception) {
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
}
