package de.hmmh.pmt;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.hmmh.pmt.db.Allocation;
import de.hmmh.pmt.db.AllocationRepository;
import de.hmmh.pmt.db.Project;
import de.hmmh.pmt.db.ProjectRepository;
import de.hmmh.pmt.employee.api.EmployeeControllerApi;
import de.hmmh.pmt.employee.api.QualificationControllerApi;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.web.servlet.MockMvc;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

@SpringBootTest
@AutoConfigureMockMvc(addFilters = false)
@ActiveProfiles("test")
public abstract class IntegrationTest {

    protected final static String baseUri = "/api/v1";
    protected final static long TEST_EMPLOYEE_A_ID = 1L;
    protected final static long TEST_QUALIFICATION_A_ID = 10L;
    protected final static long TEST_QUALIFICATION_B_ID = 11L;

    @Autowired
    protected MockMvc mvc;
    @Autowired
    protected ObjectMapper objectMapper;
    
    @Autowired
    protected ProjectRepository projectRepository;
    @Autowired 
    protected AllocationRepository allocationRepository;

    @MockBean
    protected EmployeeControllerApi mockEmployeeApi;
    @MockBean
    protected QualificationControllerApi mockQualificationApi;
    
    @BeforeEach
    void setUp() {
        allocationRepository.deleteAll();
        projectRepository.deleteAll();
    }

    @AfterEach
    void cleanUp() {
        allocationRepository.deleteAll();
        projectRepository.deleteAll();
    }

    protected Map<String, Project> createTestProjectData() {
        Map<String, Project> projects = new HashMap<>();
        
        Project researchLabProject = new Project();
        researchLabProject.setName("Underwater Research Lab");
        researchLabProject.setGoal( "Discover new marine species!");
        researchLabProject.setCustomerId(73L);
        researchLabProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        researchLabProject.setStart(LocalDateTime.of(2025, 5, 22, 8, 45));
        researchLabProject.setPlannedEnd(LocalDateTime.of(2026, 5, 22, 8, 45));
        projects.put("research-lab", researchLabProject);
        
        Project spaceStationProject = new Project();
        spaceStationProject.setName("International Space Station Expansion");
        spaceStationProject.setGoal("Build new modules for extended research.");
        spaceStationProject.setCustomerId(85L);
        spaceStationProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        spaceStationProject.setStart(LocalDateTime.of(2024, 10, 15, 10, 0));
        spaceStationProject.setPlannedEnd(LocalDateTime.of(2025, 12, 15, 10, 0));
        projects.put("space-station", spaceStationProject);
        
        Project aiResearchProject = new Project();
        aiResearchProject.setName("AI Human Interaction Study");
        aiResearchProject.setGoal("Study AI interactions in healthcare.");
        aiResearchProject.setCustomerId(63L);
        aiResearchProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        aiResearchProject.setStart(LocalDateTime.of(2023, 11, 3, 9, 30));
        aiResearchProject.setPlannedEnd(LocalDateTime.of(2024, 6, 3, 9, 30));
        projects.put("ai-research", aiResearchProject);
        
        Project renewableEnergyProject = new Project();
        renewableEnergyProject.setName("Renewable Energy Storage System");
        renewableEnergyProject.setGoal("Develop new battery tech for solar power.");
        renewableEnergyProject.setCustomerId(90L);
        renewableEnergyProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        renewableEnergyProject.setStart(LocalDateTime.of(2024, 1, 10, 11, 0));
        renewableEnergyProject.setPlannedEnd(LocalDateTime.of(2025, 1, 10, 11, 0));
        projects.put("renewable-energy", renewableEnergyProject);
        
        Project climateChangeProject = new Project();
        climateChangeProject.setName("Climate Change Impact Analysis");
        climateChangeProject.setGoal("Study global warming effects on ecosystems.");
        climateChangeProject.setCustomerId(72L);
        climateChangeProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        climateChangeProject.setStart(LocalDateTime.of(2025, 3, 12, 8, 0));
        climateChangeProject.setPlannedEnd(LocalDateTime.of(2026, 3, 12, 8, 0));
        projects.put("climate change", climateChangeProject);
        
        Project medicalResearchProject = new Project();
        medicalResearchProject.setName("Cancer Treatment Innovation");
        medicalResearchProject.setGoal("Develop new gene therapy techniques.");
        medicalResearchProject.setCustomerId(95L);
        medicalResearchProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        medicalResearchProject.setStart(LocalDateTime.of(2024, 8, 20, 9, 15));
        medicalResearchProject.setPlannedEnd(LocalDateTime.of(2026, 8, 20, 9, 15));
        projects.put("medical-research", medicalResearchProject);

        Project overlappingProjectA = new Project();
        overlappingProjectA.setName("Overlap A");
        overlappingProjectA.setGoal("A Project That Overlaps with another one for Testing");
        overlappingProjectA.setCustomerId(19L);
        overlappingProjectA.setAdministratorId(TEST_EMPLOYEE_A_ID);
        overlappingProjectA.setStart(LocalDateTime.of(1970, 6, 20, 9, 15));
        overlappingProjectA.setPlannedEnd(LocalDateTime.of(2025, 8, 20, 9, 15));
        projects.put("overlap-a", overlappingProjectA);

        Project overlappingProjectB = new Project();
        overlappingProjectB.setName("Overlap B");
        overlappingProjectB.setGoal("B Project That Overlaps with another one for Testing");
        overlappingProjectB.setCustomerId(23L);
        overlappingProjectB.setAdministratorId(TEST_EMPLOYEE_A_ID);
        overlappingProjectB.setStart(LocalDateTime.of(2024, 7, 20, 9, 15));
        overlappingProjectB.setPlannedEnd(LocalDateTime.of(2026, 12, 20, 9, 15));
        projects.put("overlap-b", overlappingProjectB);

        Project overlappingProjectC = new Project();
        overlappingProjectC.setName("Overlap C");
        overlappingProjectC.setGoal("C Project That Overlaps with another one for Testing");
        overlappingProjectC.setCustomerId(19L);
        overlappingProjectC.setAdministratorId(TEST_EMPLOYEE_A_ID);
        overlappingProjectC.setStart(LocalDateTime.of(2024, 6, 20, 9, 15));
        overlappingProjectC.setPlannedEnd(LocalDateTime.of(2026, 8, 20, 9, 15));
        projects.put("overlap-c", overlappingProjectC);


        Project nonOverlappingProject = new Project();
        nonOverlappingProject.setName("Non Overlapping to Overlap a-c");
        nonOverlappingProject.setGoal("Project That doesnt overlap with another one of the Overlap ones for Testing");
        nonOverlappingProject.setCustomerId(19L);
        nonOverlappingProject.setAdministratorId(TEST_EMPLOYEE_A_ID);
        nonOverlappingProject.setStart(LocalDateTime.of(1970, 1, 20, 9, 15));
        nonOverlappingProject.setPlannedEnd(LocalDateTime.of(1970, 2, 20, 9, 15));
        projects.put("non-overlap", nonOverlappingProject);
        
        projectRepository.saveAllAndFlush(projects.values());
        return projects;
    }


    protected Map<String, Allocation> createTestAllocationData(Map<String, Project> allProjects) {
        Map<String, Allocation> allocations = new HashMap<>();
        
        Allocation allocation1ToOverlapA = new Allocation();
        allocation1ToOverlapA.setProject(allProjects.get("overlap-a"));
        allocation1ToOverlapA.setEmployeeId(TEST_EMPLOYEE_A_ID);
        allocation1ToOverlapA.setRole(TEST_QUALIFICATION_A_ID);
        allocations.put("1>overlap-a", allocation1ToOverlapA);

        Allocation allocation1ToSpaceStation = new Allocation();
        allocation1ToSpaceStation.setProject(allProjects.get("space-station"));
        allocation1ToSpaceStation.setEmployeeId(TEST_EMPLOYEE_A_ID);
        allocation1ToSpaceStation.setRole(TEST_QUALIFICATION_A_ID);
        allocations.put("1>space-station", allocation1ToSpaceStation);

        Allocation allocation1ToAiResearch = new Allocation();
        allocation1ToAiResearch.setProject(allProjects.get("ai-research"));
        allocation1ToAiResearch.setEmployeeId(TEST_EMPLOYEE_A_ID);
        allocation1ToAiResearch.setRole(TEST_QUALIFICATION_A_ID);
        allocations.put("1>ai-research", allocation1ToAiResearch);

        allocationRepository.saveAllAndFlush(allocations.values());
        return allocations;
    }
}
