package de.hmmh.pmt.project;

import de.hmmh.pmt.IntegrationTest;
import de.hmmh.pmt.db.Allocation;
import de.hmmh.pmt.db.Project;
import de.hmmh.pmt.dtos.AddEmployeeDTO;
import de.hmmh.pmt.employee.dtos.EmployeeResponseDTO;
import de.hmmh.pmt.employee.dtos.QualificationGetDTO;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestClientException;

import java.util.List;
import java.util.Map;

import static org.mockito.ArgumentMatchers.anyLong;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class AddEmployeeTest extends IntegrationTest {

    @Test
    void addValidEmployee() throws Exception {
        when(mockEmployeeApi.findById(anyLong()))
                .thenReturn(getEmployee());

        Map<String, Project> allProjects = createTestProjectData();

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId(), getAddEmployeeDTO()))
                .andExpect(status().isNoContent());
    }

    @Test
    void shouldNotAddEmployeeWhenProjectIsNotFound() throws Exception {
        this.mvc
                .perform(getRequest(0L, getAddEmployeeDTO()))
                .andExpect(status().isNotFound());
    }

    @Test
    void shouldNotAddEmployeeWhenEmployeeDoesNotExist() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
                .thenThrow(new HttpClientErrorException(HttpStatus.NOT_FOUND));

        Map<String, Project> allProjects = createTestProjectData();

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId(), getAddEmployeeDTO()))
                .andExpect(status().isNotFound());
    }

    @Test
    void shouldReturnUnavailableWhenEmployeeApiIsDown() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
                .thenThrow(new HttpClientErrorException(HttpStatus.INTERNAL_SERVER_ERROR));

        Map<String, Project> allProjects = createTestProjectData();

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId(), getAddEmployeeDTO()))
                .andExpect(status().isServiceUnavailable());
    }

    @Test
    void shouldReturnInternalServerErrorOnApiClientCrash() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
                .thenThrow(new RestClientException("Api Client crash"));

        Map<String, Project> allProjects = createTestProjectData();

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId(), getAddEmployeeDTO()))
                .andExpect(status().isInternalServerError());
    }

    @Test
    void shouldReturnUnprocessableWhenEmployeeDoesNotHaveTheQualification() throws Exception {
        when(mockEmployeeApi.findById(anyLong()))
                .thenReturn(getEmployee());

        Map<String, Project> allProjects = createTestProjectData();

        AddEmployeeDTO addEmployeeDTO = getAddEmployeeDTO();
        addEmployeeDTO.setQualificationId(TEST_QUALIFICATION_B_ID);

        this.mvc
                .perform(getRequest(allProjects.get("research-lab").getId(), addEmployeeDTO))
                .andExpect(status().isUnprocessableEntity());
    }

    @Test
    void shouldReturnUnprocessableWhenEmployeesProjectTimeRangesOverlapsBackwards() throws Exception {
        when(mockEmployeeApi.findById(anyLong()))
                .thenReturn(getEmployee());

        Map<String, Project> allProjects = createTestProjectData();
        createTestAllocationData(allProjects);

        this.mvc
                .perform(getRequest(allProjects.get("overlap-b").getId(), getAddEmployeeDTO()))
                .andExpect(status().isUnprocessableEntity());
    }

    @Test
    void shouldReturnUnprocessableWhenEmployeesProjectTimeRangesOverlapsForwards() throws Exception {
        when(mockEmployeeApi.findById(anyLong()))
                .thenReturn(getEmployee());

        Map<String, Project> allProjects = createTestProjectData();
        createTestAllocationData(allProjects);

        this.mvc
                .perform(getRequest(allProjects.get("overlap-c").getId(), getAddEmployeeDTO()))
                .andExpect(status().isUnprocessableEntity());
    }

    @Test
    void shouldNotReturnUnprocessableWhenAllocationsDontOverlap() throws Exception {
        when(mockEmployeeApi.findById(anyLong()))
                .thenReturn(getEmployee());

        Map<String, Project> allProjects = createTestProjectData();
        createTestAllocationData(allProjects);

        this.mvc
                .perform(getRequest(allProjects.get("non-overlap").getId(), getAddEmployeeDTO()))
                .andExpect(status().isNoContent());
    }

    private QualificationGetDTO newQualification(Long id) {
        QualificationGetDTO qualificationGetDTO = new QualificationGetDTO();
        qualificationGetDTO.setId(id);
        return qualificationGetDTO;
    }

    private EmployeeResponseDTO getEmployee() {
        EmployeeResponseDTO employee = new EmployeeResponseDTO();
        employee.setId(TEST_EMPLOYEE_A_ID);
        employee.setSkillSet(List.of(newQualification(TEST_QUALIFICATION_A_ID)));
        return employee;
    }

    private AddEmployeeDTO getAddEmployeeDTO() {
        AddEmployeeDTO addEmployeeDTO = new AddEmployeeDTO();
        addEmployeeDTO.setEmployeeId(TEST_EMPLOYEE_A_ID);
        addEmployeeDTO.setQualificationId(TEST_QUALIFICATION_A_ID);
        return addEmployeeDTO;
    }

    private RequestBuilder getRequest(Long projectId, AddEmployeeDTO addEmployeeDTO) throws Exception {
        return MockMvcRequestBuilders
                .post(baseUri + "/project/" + projectId)
                .content(this.objectMapper.writeValueAsString(addEmployeeDTO))
                .contentType(MediaType.APPLICATION_JSON);
    }

}
