package de.hmmh.pmt.project;
import de.hmmh.pmt.IntegrationTest;
import de.hmmh.pmt.db.Project;
import de.hmmh.pmt.dtos.CreateProjectDTO;
import de.hmmh.pmt.employee.dtos.EmployeeResponseDTO;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.RequestBuilder;
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestClientException;
import java.time.LocalDateTime;
import java.util.Map;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

public class CreateTest extends IntegrationTest {

    @Test
    void successfullyCreate() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
                .thenReturn(new EmployeeResponseDTO());

        this.mvc
                .perform(getRequest(getCreateProjectDTO()))
                .andExpect(status().isCreated())
                .andExpect(jsonPath("$.id").exists());
    }

    @Test
    void shouldNotCreateProjectWithSameName() throws Exception {
        Map<String, Project> allProjects = createTestProjectData();
        Project spaceStation = allProjects.get("space-station");

        CreateProjectDTO createDTO = getCreateProjectDTO();
        createDTO.setName(spaceStation.getName());

        this.mvc
                .perform(getRequest(createDTO))
                .andExpect(status().isConflict());
    }

    @Test
    void shouldNotCreateProjectWhenAdministratorDoesNotExist() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
            .thenThrow(new HttpClientErrorException(HttpStatus.NOT_FOUND));

        this.mvc
                .perform(getRequest(getCreateProjectDTO()))
                .andExpect(status().isNotFound());
    }

    @Test
    void shouldReturnUnavailableWhenEmployeeApiIsDown() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
            .thenThrow(new HttpClientErrorException(HttpStatus.INTERNAL_SERVER_ERROR));

        this.mvc
            .perform(getRequest(getCreateProjectDTO()))
            .andExpect(status().isServiceUnavailable());
    }

    @Test
    void shouldReturnInternalServerErrorOnApiClientCrash() throws Exception {
        when(this.mockEmployeeApi.findById(Mockito.anyLong()))
            .thenThrow(new RestClientException("Api Client crash"));

        this.mvc
            .perform(getRequest(getCreateProjectDTO()))
            .andExpect(status().isInternalServerError());
    }

    @Test
    void shouldReturnUnprocessableWhenDataIsInvalid() throws Exception {
        CreateProjectDTO createDTO = getCreateProjectDTO();
        createDTO.setStart(LocalDateTime.of(2003, 1, 13, 12, 51));
        createDTO.setPlannedEnd(LocalDateTime.of(2002, 3, 21, 11, 42));

        this.mvc
            .perform(getRequest(createDTO))
            .andExpect(status().isUnprocessableEntity());
    }

    private CreateProjectDTO getCreateProjectDTO() {
        CreateProjectDTO createDTO = new CreateProjectDTO();
        createDTO.setName("Test");
        createDTO.setGoal("A Test Goal");
        createDTO.setCustomerId(10L);
        createDTO.setAdministratorId(10L);
        createDTO.setStart(LocalDateTime.of(2000, 1, 13, 12, 51));
        createDTO.setPlannedEnd(LocalDateTime.of(2002, 3, 21, 11, 42));
        return createDTO;
    }

    private RequestBuilder getRequest(CreateProjectDTO createDTO) throws Exception {
        return MockMvcRequestBuilders
                .post(baseUri + "/project")
                .accept(MediaType.APPLICATION_JSON)
                .content(this.objectMapper.writeValueAsString(createDTO))
                .contentType(MediaType.APPLICATION_JSON);
    }
}
